<?php

declare(strict_types=1);

namespace Drupal\cortex_socials\Form;

use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\cortex_general_settings\Form\CortexGeneralSettingsPluginForm;

/**
 * Provides a Cortex socials form.
 */
final class SocialsForm extends CortexGeneralSettingsPluginForm {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'cortex_socials_socials';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    $form = parent::buildForm($form, $form_state);
    $settings = $form_state->getBuildInfo()['args'][0];

    $uniqueId = Html::getUniqueId('social-links');
    $form['social_links'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Social Links'),
      '#tree' => TRUE,
      '#prefix' => '<div id="' . $uniqueId . '">',
      '#suffix' => '</div>',
    ];

    if (empty($form_state->getValues())) {
      $social_links = $settings['social_links']['links'] ?? [];
    }
    else {
      $social_links = $form_state->getValue(['social_links', 'links']);
    }

    $available_networks = \Drupal::config('cortex_general_settings.socials')->getOriginal('networks', FALSE);
    if (empty($available_networks)) {
      $available_networks = [];
    }
    $network_options = array_map(function ($item) {
      return $item['label'];
    }, $available_networks);

    foreach ($social_links as $key => $social_link) {
      $form['social_links']['links'][$key] = [
        '#type' => 'details',
        '#title' => $social_link['network'] ? ($social_link['network'] == 'custom' && $social_link['title'] ? $social_link['title'] : $network_options[$social_link['network']]) : $this->t('New social link'),
        '#open' => $social_link['network'] ? FALSE : TRUE,
      ];

      $form['social_links']['links'][$key]['network'] = [
        '#type' => 'select',
        '#empty_option' => $this->t('-- Select a social network --'),
        '#required' => TRUE,
        '#title' => $this->t('Social Network'),
        '#options' => $network_options,
        '#default_value' => $social_link['network'],
        '#ajax' => [
          'callback' => [$this, 'rebuildSocialLinks'],
          'wrapper' => $uniqueId,
        ],
      ];

      $form['social_links']['links'][$key]['icon'] = [
        '#type' => 'iconify_icons',
        '#title' => $this->t('Custom Network icon'),
        '#required' => TRUE,
        '#collections' => 'ri',
        '#description' => $this->t('Select an icon for the custom network.'),
        '#default_value' => $social_link['icon'] ?? '',
      ];

      $form['social_links']['links'][$key]['title'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Title'),
        '#required' => TRUE,
        '#default_value' => $social_link['title'] ?? '',
      ];

      $form['social_links']['links'][$key]['url'] = [
        '#type' => 'url',
        '#title' => $this->t('URL'),
        '#required' => TRUE,
        '#default_value' => $social_link['url'] ?? '',
      ];

      $form['social_links']['links'][$key]['delete'] = [
        '#type' => 'submit',
        '#limit_validation_errors' => [['social_links']],
        '#value' => $this->t('Delete social link'),
        '#name' => 'delete-social-link-' . $key,
        '#attributes' => ['class' => ['button', 'button--danger']],
        '#ajax' => [
          'callback' => [$this, 'rebuildSocialLinks'],
          'wrapper' => $uniqueId,
        ],
        '#submit' => [[$this, 'deleteSocialLink']],
      ];
    }

    $form['social_links']['add'] = [
      '#type' => 'submit',
      '#limit_validation_errors' => [['social_links']],
      '#name' => 'add-social-link',
      '#value' => $this->t('Add social link'),
      '#attributes' => [
        'class' => [
          'add-social-button',
          'js-add-social-button',
        ],
      ],
      '#ajax' => [
        'callback' => [$this, 'rebuildSocialLinks'],
        'wrapper' => $uniqueId,
        'event' => 'click',
      ],
      '#submit' => [[$this, 'addSocialLink']],
    ];

    return $form;
  }

  /**
   *
   */
  public function rebuildSocialLinks(array &$form, FormStateInterface $form_state): array {
    $triggeringElement = $form_state->getTriggeringElement();
    $parents = $triggeringElement['#parents'];
    $key = reset(array_splice($parents, -1, 1));
    if ($key == 'network') {
      $available_networks = \Drupal::config('cortex_general_settings.socials')->getOriginal('networks', FALSE);
      NestedArray::setValue($form, array_merge($parents, ['#open']), TRUE);
      NestedArray::setValue($form, array_merge($parents, ['title', '#value']), $available_networks[$triggeringElement['#value']]['label']);
      NestedArray::setValue($form, array_merge($parents, ['url', '#value']), $available_networks[$triggeringElement['#value']]['url']);
    }

    return NestedArray::getValue($form, ['social_links']);
  }

  /**
   *
   */
  public function deleteSocialLink(array &$form, FormStateInterface $form_state) {
    $key = reset(array_splice($form_state->getTriggeringElement()['#parents'], -2, 1));
    $form_state->setRebuild(TRUE);
    $social_links = $form_state->getValue(['social_links', 'links']) ?? [];
    unset($social_links[$key]);
    $form_state->setValue(['social_links', 'links'], $social_links);
  }

  /**
   *
   */
  public function addSocialLink(array &$form, FormStateInterface $form_state) {
    $form_state->setRebuild(TRUE);
    $social_links = $form_state->getValue(['social_links', 'links']) ?? [];
    $social_links[] = [
      'network' => '',
      'title' => '',
      'url' => '',
    ];
    $form_state->setValue(['social_links', 'links'], $social_links);
  }

}
